@@ -1,9 +1,9 @@
-# Copyright 1998-2000, Paul Johnson (pjcj@cpan.org)
+# Copyright 1998-2001, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
 # The latest version of this software should be available from my homepage:
-# http://www.pjcj.fsnet.co.uk
+# http://www.pjcj.net
 
 # documentation at __END__
 
@@ -22,7 +22,7 @@ use vars qw($VERSION $Tags);
 
 BEGIN
 {
-  $VERSION = "1.08";
+  $VERSION = "1.09";
 
   $Tags =
   {
@@ -158,10 +158,10 @@ BEGIN
   };
 }
 
-use Gedcom::Grammar    1.08;
-use Gedcom::Individual 1.08;
-use Gedcom::Family     1.08;
-use Gedcom::Event      1.08;
+use Gedcom::Grammar    1.09;
+use Gedcom::Individual 1.09;
+use Gedcom::Family     1.09;
+use Gedcom::Event      1.09;
 
 sub new
 {
@@ -169,10 +169,8 @@ sub new
   my $class = ref($proto) || $proto;
   my $self =
   {
-    buffer    => [],
     records   => [],
     tags      => $Tags,
-    tie       => 0,
     read_only => 0,
     types     => {},
     xrefs     => {},
@@ -225,7 +223,6 @@ sub new
                             callback => $self->{callback});
     $self->{record}{items} = [ Gedcom::Record->new(tag => "TRLR") ]
       unless @{$self->{record}{items}};
-
     $self->collect_xrefs;
   }
   $self;
@@ -250,13 +247,13 @@ sub write_xml
 <!--
 
 This output was generated by Gedcom.pm.
-Gedcom.pm is Copyright 1999-2000, Paul Johnson (pjcj@cpan.org)
-Version 1.08 - 8th May 2000
+Gedcom.pm is Copyright 1999-2001, Paul Johnson (pjcj@cpan.org)
+Version 1.09 - 12th February 2001
 
 Gedcom.pm is free.  It is licensed under the same terms as Perl itself.
 
 The latest version of Gedcom.pm should be available from my homepage:
-http://www.pjcj.fsnet.co.uk
+http://www.pjcj.net
 
 EOH
   $self->{fh}->print("Generated on " . localtime() . "\n\n-->\n\n");
@@ -415,7 +412,7 @@ sub get_individual
   my $ordered = sub
   {
     my ($n, @ind) = @_;
-    map { $_->[1] } grep { $_->[0] =~ $n } @ind
+    map { $_->[1] } grep { $_ && $_->[0] =~ $n } @ind
   };
 
   # search for the name in any order
@@ -444,7 +441,9 @@ sub get_individual
 
   # Store the name with the individual to avoid continually recalculating it.
   # This is a bit like a Schwartzian transform, with a grep instead of a sort.
-  my @ind = map { [ $_->tag_value("NAME") => $_ ] } @individuals;
+  my @ind =
+    map { [ do { my $n = $_->tag_value("NAME"); defined $n ? $n : "" } => $_ ] }
+    @individuals;
 
   for my $n ( map { qr/^$_$/, qr/\b$_\b/, $_ } map { $_, qr/$_/i } qr/\Q$name/ )
   {
@@ -493,7 +492,7 @@ __END__
 
 Gedcom - a module to manipulate Gedcom genealogy files
 
-Version 1.08 - 8th May 2000
+Version 1.09 - 12th February 2001
 
 =head1 SYNOPSIS
 
@@ -502,9 +501,11 @@ Version 1.08 - 8th May 2000
   my $ged = Gedcom->new(gedcom_file => $gedcom_file);
   my $ged = Gedcom->new(grammar_version => 5.5,
                         gedcom_file     => $gedcom_file,
+                        read_only       => 1,
                         callback        => $cb);
   my $ged = Gedcom->new(grammar_file => "gedcom-5.5.grammar",
                         gedcom_file  => $gedcom_file);
+
   return unless $ged->validate;
   my $xref = $self->resolve_xref($value);
   $ged->resolve_xrefs;
@@ -521,12 +522,12 @@ Version 1.08 - 8th May 2000
 
 =head1 DESCRIPTION
 
-Copyright 1998-2000, Paul Johnson (pjcj@cpan.org)
+Copyright 1998-2001, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
 The latest version of this software should be available from my homepage:
-http://www.pjcj.fsnet.co.uk
+http://www.pjcj.net
 
 This module provides for manipulation of Gedcom files.  Gedcom is a
 format for storing genealogical information designed by The Church of
@@ -763,6 +764,12 @@ provided that not too much of the gedcom file is needed.  If the whole
 of the gedcom file needs to be read, for example to validate it, or to
 write it out in a different format, then this option should not be used.
 
+When using the read_only option an index file is kept which can also
+speed up operations.  It's usage should be transparant, but will require
+write access to the directory containing the gedcom file.  If you access
+individuals only by their xref (eg I20) then the index file will allow
+only the relevant parts of the gedcom file to be read.
+
 With or without the read_only option, the gedcom file is accessed in the
 same fashion and the data structures can be changed.  In this respect,
 the name read_only is not very accurate, but since changing the Gedcom
@@ -770,11 +777,11 @@ data will generally mean that the data will be written which means that
 the data will first be read, the read_only option is generally useful
 when the data will not be written and when not all the data will be
 read.  You may find it useful to experiment with this option and check
-the amount of CPU time and memroy that your application uses.
+the amount of CPU time and memory that your application uses.
 
 callback is an optional reference to a subroutine which will be called
-at various times while the gedcom file (and the grammar file, ir
-applicable) is being read.  It's purpose is to provide feedback during
+at various times while the gedcom file (and the grammar file, if
+applicable) is being read.  Its purpose is to provide feedback during
 potentially long operations.  The subroutine is called with five
 arguments:
 
